{$X+}
{
                 DBBASE III+ Support component by Daniel Parnell

  I wrote the following code over a couple of hours because I noticed that
people were complaining in comp.lang.pascal (usenet newsgroup) that to use
any data base files in their applications made their resulting programs huge.
So, I grabbed my Delphi books and had a go at creating a DBASE III+ unit, and
here is the result.

  Now for the legal stuff.  Anybody can use the following code for anything
that they want.  I would however like to be notified of any use this code is
put to because I'm interested in seeing what is done with it.  Credit would
be nice also.

Note: These routines ignore the record deletion flag.

                                Daniel
                     s921878@minyos.xx.rmit.edu.au
}

unit DBase3P;

interface

uses Dialogs,Classes,UCBaza;

const
  MaxSize = $7FFF; { Size of the record buffer }

type
  TBigArray = array[0..MaxSize] of char;
  PBigArray = ^TBigArray;

  THeader = record
    version        : byte;    { Should be 3 or $83 }
    year,month,day : byte;    { Date of last update }
    numRecs        : longint; { Number of records in the file }
    headLen        : word;    { Length of the header }
    recLen         : word;    { Length of individual records }
    rez            : array[0..19] of byte; { reserved }
  end;

  PField = ^TField;
  TField = record
    name   : array[0..10] of char;  { Name of the field }
    what   : char;                  { Type of data in this field }
    data   : array[0..1] of word;   { Not used }
    len    : byte;                  { Length of the field }
    places : byte;                  { Number of decimal places }
    rez    : array[0..13] of byte;  { Reserved data area }
  end;

  TDBase3Plus = class(TCustomBaza)
  private
    fileOpen  : boolean;   { TRUE if the file is open }
    DBfile    : file;      { The actual file }
    buffer    : PBigArray; { Temp buffer for record }

  protected
    procedure SetFileName(name : string); override;
  public
    about     : THeader;         { Info about the file }

    constructor Create;
    destructor Destroy; override;

    procedure Close; override;
    function Open:boolean; override;
    procedure write(r : longint); override;
    procedure Seek(r : longint); override;
    procedure NewRecord; override;
    procedure GotoStart; override;
    procedure GotoEnd; override;
    procedure GotoNext; override;
  public
    property FileName;
  end;

implementation
uses SysUtils;


constructor TDBase3Plus.Create;
begin
  inherited Create;
  fileOpen:=FALSE;
  fileName:='';
  buffer:=nil;
  fields:=tStringList.Create;
  fields.sorted:=false;
  rec:=tStringList.Create;
  rec.sorted:=false;
end;

procedure TDBase3Plus.SetFileName(name : string);
begin
  if fileOpen and (FfileName<>name) then
    close;

  FfileName:=name;
end;

function TDBase3Plus.open;
var
  temp : TField;
  pole: pField;
  done : boolean;
  pfm: byte;

  procedure Kom1;
  begin
    MessageDlg('Bd w bazie danych lub baza nie ma struktury DBASE 3 PLUS',mtError,[mbCancel],0);
  end;

begin
  if FileOpen then
    close; { Close the file if it already open }

  done:=FALSE;

  Result:=false;
  buffer:=nil;
  if fFileName<>'' then
  begin
    try
      assignFile(DBfile,FfileName);
      pfm:=fileMode;
      fileMode:=0;
      {$I-} reset(DBfile,1); {$I+}
      fileMode:=pfm;
      if ioResult=0 then
      begin
        try
          blockRead(DBFile,about,sizeof(THeader)); { Get the header }

          getMem(buffer,about.recLen); { Get some memory for the buffer }

          repeat
            blockRead(DBFile,temp,sizeof(TField));
            if temp.name[0]<>#$0D then
              begin
                New(pole);
                pole^:=temp;
                fields.AddObject(StrPas(pole^.name),pointer(pole));
              end
            else
              done:=TRUE;
          until DONE;

          seek(0);
          Result:=true;
          FileOpen:=TRUE;
        except
          Close;
          Kom1;
        end;
      end;
    except
      Kom1;
    end;
  end;
end;

procedure TDBase3Plus.close;
var
  f: integer;
begin
  if fileOpen then
  begin
    FileOpen:=FALSE;
    closeFile(DBfile);
    if buffer<>nil then
      freeMem(buffer,about.recLen);
    buffer:=nil;
    for f:=0 to fields.count-1 do
      Dispose(pField(fields.objects[f]));
    fields.Clear;
  end;
end;

procedure TDBase3Plus.write(r : longint);
var
  ad  : word;
  i,j : integer;
  s   : string;

begin
  cRecord:=r;
  system.seek(DBFile,r*about.recLen+about.headLen);

  buffer^[0]:=' '; { Record not deleted! }
  ad:=1;  { Skip over the deletion flag }
  for i:=0 to fields.count-1 do
    begin
      s:=rec[i];

      while length(s)<pField(fields.objects[i])^.len do
        s:=s+' ';

      for j:=1 to length(s) do
        begin
          buffer^[ad]:=s[j];
          inc(ad);
        end;
    end;

  blockWrite(DBFile,buffer^,about.recLen);
end;

procedure TDBase3Plus.seek(r : longint);
var
  ad  : word;
  i,j : integer;
  s   : string;
  l: extended;
  code: integer;
begin
  cRecord:=r;
  {$I-} system.seek(DBFile,r*about.recLen+about.headLen); {$I+}
  if ioResult=0 then
    {$I-} blockRead(DBFile,buffer^,about.recLen); {$I+}
  if ioResult=0 then
  begin
    rec.Clear;
    ad:=1; { Ignore the deletion flag }
    for i:=0 to fields.count-1 do
    begin
      s:='';
      for j:=1 to pField(fields.objects[i])^.len do
        begin
          s:=s+buffer^[ad];
          inc(ad);
        end;

      while (length(s)>0) and (Copy(s,length(s),1)=' ') do
        s:=copy(s,1,length(s)-1);
      while (length(s)>0) and (Copy(s,1,1)=' ') do
        System.Delete(s,1,1);
      if pField(fields.objects[i])^.what='N' then
      begin
        Val(s,l,code);
        Str(l:0:pField(fields.objects[i])^.places,s);
      end;
      if pField(fields.objects[i])^.what='F' then
      begin
        Val(s,l,code);
        Str(l:0:pField(fields.objects[i])^.places,s);
      end;

      rec.Add(s);
    end;
  end else
    BazaEOF:=Eof(DbFile);
end;

procedure TDBase3Plus.GotoStart;
begin
  seek(0);
end;

procedure TDBase3Plus.GotoEnd;
begin
  seek(about.numRecs-1);
end;

procedure TDBase3Plus.GotoNext;
begin
  seek(cRecord+1);
end;

procedure TDBase3Plus.NewRecord;
begin
  rec.Clear;

  inc(about.numRecs);
  system.seek(DBFile,0);
  blockWrite(DBFile,about,sizeof(THeader));
  write(about.numRecs-1);
  seek(about.numRecs-1);
end;

destructor TDBase3Plus.Destroy;
var
  f: Integer;
begin
  for f:=0 to fields.count-1 do
    Dispose(pfield(fields.objects[f]));
  fields.Clear;

  inherited Destroy;
end;

end.
