unit Upowiaz;

interface
uses Classes;
type
  tPowiazania = class(tObject)
  private
    fNr: integer;
    procedure SetNr(value:integer);
    function DajNSekcji:string;
    function DajNPlIni:string;
    function GetNPola(nrPola:integer; index:integer):string;
  public
    listaPol: tStringList;
    nazwa: string;
    symbol: string[20];
    nBazy: string;
    nPola1,nPola2,nPola3: string[40];
    przeliczaj:boolean;
    kodyZn: byte;
    typBazy: Byte;
    constructor Create;
    destructor Destroy; override;
    property Nr:integer read fNr write SetNr;
    procedure Wczytaj;
    procedure Zapisz;
    procedure Dopisz(nPolaFormul,nPolaBazy:openString);
    procedure Zeruj;
    procedure DajListeNazw(const lista:tStrings);
    function IlePowiazan:integer;
    property nPolaformul[nrPola:integer]:string index 1 read GetNPola;
    property nPolaBazy[nrPola:integer]:string index 2 read GetNPola;
    function NaWin(tx:openString):string;
  end;

implementation
uses WinProcs,IniFilEx, SysUtils;

constructor tPowiazania.Create;
begin
  inherited Create;
  listaPol:=tStringList.Create;
end;

destructor tPowiazania.Destroy;
begin
  listaPol.Free;
  inherited Destroy;
end;

function tPowiazania.DajNPlIni;
begin
  Result:=ExtractfilePath(IniFilEx.DajNPlIni)+'powiaz.dta';
end;

procedure tPowiazania.SetNr;
begin
  if value<>fNr then
  begin
    fNr:=value;
    if fNr<>0 then
      Wczytaj
    else
      Zeruj;
  end;
end;

function tPowiazania.DajNSekcji;
begin
  Result:='Def'+LoadZero(IntToStr(fNr),3);
end;

procedure tPowiazania.Zapisz;
var
  plIni: tIniFileEx;
  lista: tStringList;
  f: integer;
  sekcja: string[20];
  n: integer;
begin
  plIni:=tIniFileEx.Create(DajNPlIni);
  if fNr=0 then
  begin
    lista:=tStringList.Create;
    plIni.ReadSections(lista);
    lista.Sort;
    for f:=lista.count-1 downto 0 do
      if Copy(lista.strings[f],1,3)='Def' then
      begin
        n:=StrToNum(Copy(lista.strings[f],4,3));
        if n>0 then
        begin
          fNr:=n+1;
          Break;
        end;
      end;
    lista.Free;
  end;
  if fNr=0 then
    fNr:=1;

  sekcja:=DajNSekcji;
  plIni.EraseSection(sekcja);
  plIni.WriteString(sekcja,'nazwa',nazwa);
  plIni.WriteString(sekcja,'symbol',symbol);
  plIni.WriteString(sekcja,'nBazy',nBazy);
  plIni.WriteString(sekcja,'nPola1',nPola1);
  plIni.WriteString(sekcja,'nPola2',nPola2);
  plIni.WriteString(sekcja,'nPola3',nPola3);
  plIni.WriteBool(sekcja,'_przelicz',przeliczaj);
  plIni.WriteInteger(sekcja,'kodyZn',kodyZn);
  plIni.WriteInteger(sekcja,'typ',typBazy);
  for f:=0 to listaPol.count-1 do
    plIni.WriteString(sekcja,'p'+DajKlucz(listaPol.strings[f]),DajValue(listaPol.strings[f]));
  plIni.Free;
end;

procedure tPowiazania.Wczytaj;
var
  plIni: tIniFileEx;
  lista: tStringList;
  tx: string;
  i,f: integer;
begin
  if fNr<>0 then
  begin
    lista:=tStringList.Create;
    plIni:=tIniFileEx.Create(DajNPlIni);
    plIni.ReadSectionValues(DajNSekcji,lista);
    plIni.Free;

    nazwa:=lista.values['nazwa'];
    symbol:=lista.values['symbol'];
    nBazy:=lista.values['nBazy'];
    nPola1:=lista.values['nPola1'];
    nPola2:=lista.values['nPola2'];
    nPola3:=lista.values['nPola3'];
    przeliczaj:=lista.values['_przelicz']='1';
    kodyZn:=StrToNum(lista.values['kodyZn']);
    if kodyZn=0 then
      kodyZn:=1;
    typBazy:=StrToNum(lista.values['typ']);

    listaPol.Clear;
    i:=0;
    for f:=0 to lista.count-1 do
    begin
      tx:=lista.strings[f];
      if Copy(tx,1,1)='p' then
      begin
        listaPol.Add(Copy(tx,2,length(tx)));
        Inc(i);
      end;
    end;

    lista.Free;
  end;
end;

procedure tPowiazania.Dopisz;
begin
  if nPolaFormul<>'' then
    listaPol.Values[nPolaFormul]:=nPolaBazy;
end;

procedure tPowiazania.Zeruj;
begin
  listaPol.Clear;
  nazwa:='';
  nBazy:='';
  symbol:='';
  nPola1:='';
  nPola2:='';
  nPola3:='';
end;

procedure tPowiazania.DajListeNazw;
const
  wlkBuf = 32000;
var
  lista0: tStringList;
  f,nr: integer;
  plIni: tIniFileEx;
  buf: pChar;
  buf2: array[0..255] of char;
begin
  if lista<>nil then
  begin
    lista0:=tStringList.Create;
    plIni:=tIniFileEx.Create(DajNPlIni);
    plIni.ReadSections(lista0);
    for f:=0 to lista0.count-1 do
    begin
      if copy(lista0.strings[f],1,3)='Def' then
      begin
        nr:=StrToNum(Copy(lista0.strings[f],4,3));
        lista.AddObject(plIni.ReadString(lista0.strings[f],'nazwa',''),pointer(nr));
      end;
    end;
    plIni.Free;
    lista0.Free;
  end;
end;

function tPowiazania.IlePowiazan;
begin
  Result:=listaPol.count;
end;

function tPowiazania.GetNPola;
begin
  Result:='';
  if (nrPola>0) and (nrPola<=listaPol.count) then
    Case index of
      1: Result:=DajKlucz(listaPol.strings[nrPola-1]);
      2: Result:=DajValue(listaPol.strings[nrPola-1]);
    end;
end;

function tPowiazania.NaWin;
var
  f,dl: word;
begin
  dl:=length(tx);
  Case kodyZn of
    3:  {852}
      for f:=1 to length(tx) do
        Case tx[f] of
          #164: tx[f]:=#165;
          #143: tx[f]:=#198;
          #168: tx[f]:=#202;
          #157: tx[f]:=#163;
          #227: tx[f]:=#209;
          #224: tx[f]:=#211;
          #151: tx[f]:=#140;
          #141: tx[f]:=#143;
          #189: tx[f]:=#175;
          #165: tx[f]:=#185;
          #134: tx[f]:=#230;
          #169: tx[f]:=#234;
          #136: tx[f]:=#179;
          #228: tx[f]:=#241;
          #162: tx[f]:=#243;
          #152: tx[f]:=#156;
          #171: tx[f]:=#159;
          #190: tx[f]:=#191;
        end;

    2:  {Mazowia}
      for f:=1 to dl do
        Case tx[f] of
          #143: tx[f]:=#165;
          #149: tx[f]:=#198;
          #144: tx[f]:=#202;
          #156: tx[f]:=#163;
          #165: tx[f]:=#209;
          #163: tx[f]:=#211;
          #152: tx[f]:=#140;
          #160: tx[f]:=#143;
          #161: tx[f]:=#175;
          #134: tx[f]:=#185;
          #141: tx[f]:=#230;
          #145: tx[f]:=#234;
          #146: tx[f]:=#179;
          #164: tx[f]:=#241;
          #162: tx[f]:=#243;
          #158: tx[f]:=#156;
          #166: tx[f]:=#159;
          #167: tx[f]:=#191;
        end;
  end;
  Result:=tx;
end;


end.
